----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:17:42 05/02/2010 
-- Design Name: 
-- Module Name:    pppcmcia - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity pppcmcia is
	Port (CLK	: in		STD_LOGIC;
			
			nCE	: in		STD_LOGIC;
			nOE	: in		STD_LOGIC;
			nIOW	: in		STD_LOGIC;
			nIOR	: in		STD_LOGIC;
			nREG	: in		STD_LOGIC;
			A		: in		STD_LOGIC_VECTOR(4 downto 1);
			D		: inout	STD_LOGIC_VECTOR(7 downto 0);
			nWAIT	: out		STD_LOGIC;
			nINPA	: out		STD_LOGIC;
			
			PIOA	: inout	STD_LOGIC_VECTOR(7 downto 0);
			PIOAD	: out		STD_LOGIC;
			PIOB	: inout	STD_LOGIC_VECTOR(3 downto 0);
			LED	: out		STD_LOGIC);
end pppcmcia;

architecture Behavioral of pppcmcia is

	constant REG_ID_CFG : integer := 0;
	constant REG_ID_PIOA_D : integer := 1;
	constant REG_ID_PIOB_D : integer := 2;
	constant REG_ID_DELAY : integer := 3;

	signal cis_d : std_logic_vector(7 downto 0);
	signal reg_cfg : std_logic_vector(5 downto 0) := "1111" & "1" & "0";	-- default: sve=input, led=off
	signal reg_piob_d, shadow_piob_io : std_logic_vector(PIOB'range);
	signal shadow_read, reg_pioa_d : std_logic_vector(7 downto 0);
	signal reg_delay : std_logic_vector(7 downto 0) := x"00";
	signal shadow_pioa_io : std_logic;
	
	signal addr : integer;
	
	signal wait_start : std_logic;
	
begin

shadow_piob_io <= reg_cfg(5 downto 2);
shadow_pioa_io <= reg_cfg(1);
LED <= reg_cfg(0);	-- led je kao shadow

addr <= conv_integer(A(2 downto 1));	-- ne trebamo sve adrese za registre

-- pioa, piob
PIOAD <= not shadow_pioa_io;
PIOA <= reg_pioa_d when shadow_pioa_io='0' else (others => 'Z');

PIOBG: for i in PIOB'range generate
	PIOB(i) <= reg_piob_d(i) when shadow_piob_io(i)='0' else 'Z';
end generate;

-- cis
cisrom0: entity work.CIS_ROM generic map (filename=>"cis.rom")
										port map (A => A, D => cis_d);

-- read ack
nINPA <= '0' when (nIOR='0' or nOE='0') and nCE='0' else '1';

-- prikaz registra ili cis-a
D <=	shadow_read			when nCE='0' and nIOR='0' else
		cis_d					when nCE='0' and nOE='0' and nREG='0' else
		(others => '0')	when nCE='0' and nOE='0' else
		(others => 'Z');

-- odabir registra za prikaz
with addr select
	shadow_read <=	"--" & reg_cfg			when REG_ID_CFG,
						PIOA						when REG_ID_PIOA_D,
						"0000" & PIOB			when REG_ID_PIOB_D,
						reg_delay				when REG_ID_DELAY,
						(others => '-')		when others;

-- zapis registara
process (nIOW)
begin
	if (falling_edge(nIOW) and nCE='0') then
		case addr is
			when REG_ID_CFG		=> reg_cfg <= D(reg_cfg'range);
			when REG_ID_PIOA_D	=> reg_pioa_d <= D;
			when REG_ID_PIOB_D	=> reg_piob_d <= D(reg_piob_d'range);
			when REG_ID_DELAY	=> reg_delay <= D;
			when others => null;
		end case;
	end if;
end process;

-- cekanje prilikom pisanja po PIOx_D
wait_start <= '1' when (addr = REG_ID_PIOA_D or addr = REG_ID_PIOB_D) and nIOW='0' else '0';
waiter0: entity work.monostable_delay port map(clk => CLK, value => reg_delay, start => wait_start, state => nWAIT);

end Behavioral;

